﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using ThinkGeo.Core;

namespace WpfDotNetFramwork
{
    public enum Position { UpperLeft, UpperRight, LowerLeft, LowerRight };

    //This class accepts two images. One image for the needle (north arrow) and another optional image for  
    //the frame of the compass. Typically, we want to have an image for the frame representing the angles (in degrees or other units)  
    //representing to what angle the map is rotated. We could have a regular frame as in a traditional compass.  
    //The image for the frame stays fixed as the map rotates whereas the image for the needle rotates with the map to always point  
    //to the north. Keep in mind that the frame image needs to have to transparency so that it does not occult the needle image.  

    class CompassAdornmentLayer : AdornmentLayer
    {
        GeoImage needleImage = null;
        GeoImage frameImage = null;
        Position position;
        int sizePercentage;
        float rotateAngle;

        //Use that constructor with only one image for the needle (will work as a simple north arrow)  
        public CompassAdornmentLayer(GeoImage needleImage, Position position, int sizePercentage)
        {
            this.needleImage = needleImage;
            this.position = position;
            this.sizePercentage = sizePercentage;
        }

        //Use that constructor with both images for needle and frame of the compass.  
        public CompassAdornmentLayer(GeoImage needleImage, GeoImage frameImage, Position position, int sizePercentage)
        {
            this.needleImage = needleImage;
            this.frameImage = frameImage;
            this.position = position;
            this.sizePercentage = sizePercentage;
        }

        public GeoImage NeedleImage
        {
            get { return needleImage; }
            set { needleImage = value; }
        }

        public GeoImage FrameImage
        {
            get { return frameImage; }
            set { frameImage = value; }
        }

        //We needs to update that property as the map rotates to have the needle showing the north accurately.  
        public float RotateAngle
        {
            get { return rotateAngle; }
            set { rotateAngle = value; }
        }

        public Position Position
        {
            get { return position; }
            set { position = value; }
        }

        public int SizePercentage
        {
            get { return sizePercentage; }
            set { sizePercentage = value; }
        }

        protected override void DrawCore(GeoCanvas canvas, Collection<SimpleCandidate> labelsInAllLayers)
        {
            //Draws the scaled images of needle and frame images according to the percentage size. For example with a PercentageSize of 80,  
            //the compass size will be 80% of the size of needle image.  

            //Sets the position of the compass.  
            int margin = 10;
            int Xloc = 0;
            int Yloc = 0;
            int imageWidth = (sizePercentage * needleImage.Width) / 100;
            int imageHeight = (sizePercentage * needleImage.Height) / 100;
            switch (position)
            {
                case Position.UpperLeft:
                    Xloc = (imageWidth / 2) + margin;
                    Yloc = (imageHeight / 2) + margin;
                    break;
                case Position.UpperRight:
                    Xloc = ((int)canvas.Width - (imageWidth / 2)) - margin;
                    Yloc = (imageHeight / 2) + margin;
                    break;
                case Position.LowerLeft:
                    Xloc = (imageWidth / 2) + margin;
                    Yloc = ((int)canvas.Height - (imageHeight / 2)) - margin;
                    break;
                case Position.LowerRight:
                    Xloc = ((int)canvas.Width - (imageWidth / 2)) - margin;
                    Yloc = ((int)canvas.Height - (imageHeight / 2)) - margin;
                    break;
            }

            //Draws the needle image at the size according to the percentage size and rotate the image according to RotateAngle property.  
            canvas.DrawScreenImage(needleImage, Xloc, Yloc, imageWidth, imageHeight, DrawingLevel.LevelFour, 0, 0, rotateAngle);
            //If the frame image has been set, draws image at the size according to the percentage size with no rotation.  
            if (frameImage != null)
            {
                canvas.DrawScreenImage(frameImage, Xloc, Yloc, imageWidth, imageHeight, DrawingLevel.LevelFour, 0, 0, 0);
            }
        }
    }
}
